//! Traits that construct SELECT statements
//!
//! Traits in this module have methods that generally map to the keyword for the corresponding clause in SQL,
//! unless it conflicts with a Rust keyword (such as `WHERE`/`where`).
//!
//! See also [`expression_methods`][expression_methods] and [`dsl`][dsl].
//!
//! [expression_methods]: ../expression_methods/index.html
//! [dsl]: ../dsl/index.html
mod belonging_to_dsl;
#[doc(hidden)]
pub mod boxed_dsl;
mod count_dsl;
mod distinct_dsl;
mod group_by_dsl;
mod join_dsl;
#[doc(hidden)]
pub mod limit_dsl;
#[doc(hidden)]
pub mod load_dsl;
mod locking_dsl;
#[doc(hidden)]
pub mod select_dsl;
#[doc(hidden)]
pub mod filter_dsl;
mod save_changes_dsl;
mod offset_dsl;
mod order_dsl;

pub use self::belonging_to_dsl::BelongingToDsl;
pub use self::boxed_dsl::BoxedDsl;
pub use self::count_dsl::CountDsl;
pub use self::distinct_dsl::DistinctDsl;
#[cfg(feature = "postgres")]
pub use self::distinct_dsl::DistinctOnDsl;
pub use self::filter_dsl::{FilterDsl, FindDsl};
#[doc(hidden)]
pub use self::group_by_dsl::GroupByDsl;
pub use self::join_dsl::{InternalJoinDsl, JoinDsl, JoinOnDsl, JoinWithImplicitOnClause};
pub use self::limit_dsl::LimitDsl;
pub use self::load_dsl::{ExecuteDsl, FirstDsl, LoadDsl, LoadQuery};
pub use self::locking_dsl::ForUpdateDsl;
pub use self::offset_dsl::OffsetDsl;
pub use self::order_dsl::OrderDsl;
pub use self::save_changes_dsl::SaveChangesDsl;
pub use self::select_dsl::SelectDsl;
